import { $, camelize, isJQuery, isPlainObject, isString, toNode } from '../util/index';

export default function (UIkit) {

    const DATA = UIkit.data;

    UIkit.components = {};

    UIkit.component = function (id, options) {

        var name = camelize(id);

        if (isPlainObject(options)) {
            options.name = name;
            options = UIkit.extend(options);
        } else {
            options.options.name = name;
        }

        UIkit.components[name] = options;

        UIkit[name] = function (element, data) {

            if (isPlainObject(element)) {
                return new UIkit.components[name]({data: element});
            }

            if (UIkit.components[name].options.functional) {
                return new UIkit.components[name]({data: [...arguments]});
            }

            data = data || {};
            element = isString
                ? $(element)[0]
                : isJQuery(element)
                    ? element[0]
                    : element;

            return element && element[DATA] && element[DATA][name] || new UIkit.components[name]({el: element, data});
        };

        if (document.body && !options.options.functional) {
            UIkit[name](`[uk-${id}],[data-uk-${id}]`);
        }

        return UIkit.components[name];
    };

    UIkit.getComponents = element => element && toNode(element)[DATA] || {};
    UIkit.getComponent = (element, name) => element && UIkit.getComponents(element)[name];

    UIkit.connect = node => {

        var name;

        if (node[DATA]) {
            for (name in node[DATA]) {
                node[DATA][name]._callConnected();
            }
        }

        for (var i = 0; i < node.attributes.length; i++) {

            name = node.attributes[i].name;

            if (name.lastIndexOf('uk-', 0) === 0 || name.lastIndexOf('data-uk-', 0) === 0) {

                name = camelize(name.replace('data-uk-', '').replace('uk-', ''));

                if (UIkit[name]) {
                    UIkit[name](node);
                }
            }
        }

    };

    UIkit.disconnect = node => {
        for (var name in node[DATA]) {
            node[DATA][name]._callDisconnected();
        }
    }

}
